/* GETPATH.C - General purpose file name finder (uses args or -z) 
               This function is used in conjunction with the 
               argparse functions (getopt, getarg). It returns a 
               pointer to a filename or NULL. 
                    
               If the listflag parameter is set it reads filenames, 
               one per line, from 'pathlist'. Otherwise they are 
               taken from the command line. 
                    
               Assumptions are that:
                    
               -    getopt() has processed the command line before
                    it is called, 
               -    pathlist is a file that is open when getpath()
                    is called if listflag==TRUE,
               -    optind has been reset to 1, 
               -    all non-option arguments are filenames. 
                    
*/

#include <stdio.h>
#include <ctype.h>
#include "environ.h"

#ifdef OS9
#include <strings.h>
#else
#include <string.h>
#endif

char* getpath(int argc, char *argv[], FILE *pathlist, int listflag)
{
    int            r;
    static char    nextpath[256];
    int            quote;
    int            c;
    int            i;
    
    void _fillstr(FILE *p, char *s, int t, int n);
    
    if (listflag)                                 /* next name */
    {                                             /* from file */
        while (isspace((c = getc(pathlist))))
            ;
        if (c != EOF)
        {
            switch(c)
            {
                case '"':
                case '\'':
                    _fillstr(pathlist,nextpath,c,255);
                    break;
                default:
                    ungetc(c,pathlist);
                    _fillstr(pathlist,nextpath,BLANK,255);
            }
            return(nextpath);
        }
    }
    else                                          /* next param */
    {                                             /* from cmdline */
        while (!(r = getarg(argc,argv)));          /* skip -opts */
        if (r != EOF)                             /* get filename */ 
            return(optarg);
    }
    return((char*)NULL);                          /* nothing found */
}
            
void _fillstr(FILE *p, char *s, int t, int n)
{
    int     run;
    int     c;
    char    *err;

    err = s;
    run = TRUE;
    while(run && n)
    {
        c = getc(p);
        if (t != BLANK)
        {
            *s = 0;
            if (c == EOF || c == '\n')
                error(progname(),"Unterminated string in file list:",err);
        }
        if (t == BLANK && (isspace(c) || c == EOF))
            run = FALSE;
        else
        if (t != BLANK && (c == t || c == EOF))
            run = FALSE;
        else
        {
            *s++ = c;
            n--;
        }
    }
    *s = 0;
}
