/* DIRACC.C - Portable directory access module.    */

#include "environ.h"

#ifdef OS9
#include <strings.h>
#include <modes.h>
#endif

#ifdef MSDOS
#include <string.h>
#include <sys\stat.h>
#include <io.h>
#include <stdlib.h>
#include <fcntl.h>
#include <dir.h>
#include <dos.h>
#endif

#ifdef LINUX
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>
#include <stdlib.h>
#endif

#ifdef OS9

/*
    OS9/68K-specific versions of diropen, dirclose, dirread, dirsize.
*/

DIRPATH *diropen(char *p)    
{
    DIRPATH *dp;
    FILE    *fp;

    fp = fopen(p,"d");       /* open as directory */
    if (fp == NULL)          /* didn't open */
        return(NULL);

    /*
        Now we've got the directory open, set up the
        DIRPATH control block
    */

    dp = (DIRPATH*)calloc(1,sizeof(DIRPATH)); 
    if (dp == NULL)
        return(NULL);        /* couldn't get the space */
    dp->p = fp;              /* keep the directory file pointer */
    strcpy(dp->path,p);      /* and the path name */
    return(dp);
}

int dirclose(dp)
DIRPATH *dp;
{
    int temp;
    temp = fclose(dp->p);    /* close the directory */
    free(dp);                /* ditch the DIRPATH control block */
    return(temp);           /* return the fclose reply */
}

int dirread(dp,name)
DIRPATH *dp;
char    *name;
{
    int     r,a,i;
    char    fullname[NAMELTH];
    BYTE    temp[DIRENT];
    
    if (fread(temp,DIRENT,1,dp->p) == 0)
        r = DIR_EOF;
    else
    {
        for (i = 0; i < NAMESIZE && temp[i]; i++) 
        {
            if (temp[i] >= 0x80) 
                name[i] = temp[i] - 0x80;
            else
                name[i] = temp[i];
        }
        name[i] = NULL;
        strcpy(fullname,dp->path);
        strcat(fullname,PATHSEP);
        strcat(fullname,name);
        if (name[0] == NULL)
            r = DIR_NULL;
        else
        {
            a = access(fullname,S_IFDIR);
            if (a == 0)
                r = DIR_SUBDIR;
            else
                r = DIR_FILE;    
        }
    }
    return(r);
}

#endif        

#ifdef MSDOS

/* 
    MS-DOS-specific versions of diropen, dirclose, dirread, dirsize.
*/

DIRPATH *diropen(p)    
char *p;
{
    DIRPATH      *dp;
    char         temp[NAMELTH];
    int          i;

    struct stat  statbuf;
    struct ffblk *fb;

    if (strlen(p) > (NAMELTH - 5))
        return(NULL);
    strcpy(temp,p);
    i = strlen(temp);
    if (i == 0)
        return(NULL);              /* no name supplied */
    if (temp[i-1] == '\\')
        temp[--i] = NULL;          /* remove trailing slash */
    if (i == 0)
        return(NULL);              /* only slash supplied */
    if (temp[i-1] != ':')
    {                              /* omit the following for devices */
        if (access(temp,0))
            return(NULL);          /* no such file */
        if (stat(temp,&statbuf))           
            return(NULL);          /* stat fails to find stuff */
        if (statbuf.st_mode & S_IFREG)
            return(NULL);          /* not a directory */
    }

    /*
        The path is known to point to a valid directory, so
        set up the controlling ffblk (needed by 'findfirst'
        and 'findnext') and the DIRPATH control block.
    */

    fb = (struct ffblk*)calloc(1,sizeof(struct ffblk));
    if (fb == NULL)
        return(NULL);              /* out of space */
    dp = (DIRPATH*)calloc(1,sizeof(DIRPATH)); 
    if (dp == NULL)
        return(NULL);              /* ditto */
    dp->p = fb;                    /* pointer to find... control block */
    dp->status = FIRSTCALL;        /* preset first time flag */
    strcat(temp,PATHSEP);          /* add the path separator */
    strcat(temp,"*.*");            /* and the search-all wildcard */
    strcpy(dp->path,temp);         /* save the path name */

    return(dp);
}

int dirclose(dp)
DIRPATH *dp;
{
    free(dp->p);
    dp->p = NULL;
    free(dp);
    return(NULL);
}

int dirread(dp,name)        
DIRPATH *dp;
char    *name;
{
    int     r,
            i,
            whatfound;

    if (dp->status == FIRSTCALL)
    {
        r = findfirst(dp->path,dp->p,FA_DIREC);
        dp->status = NEXTCALL;
    }
    else
        r = findnext(dp->p);

    if (r)
        whatfound = DIR_EOF;
    else
    {
        for (i = 0; i < NAMESIZE && dp->p->ff_name[i]; i++)
            name[i] = dp->p->ff_name[i];
        name[i] = NULL;
        if (dp->p->ff_attrib & FA_DIREC)
            whatfound = DIR_SUBDIR;
        else
            whatfound = DIR_FILE;
    }
    return(whatfound);
}        
#endif        

#ifdef LINUX

/*
    Linux-specific versions of diropen, dirclose, dirread, dirsize.
*/

DIRPATH *diropen(char *p)    
{
    DIRPATH *dp;
    DIR     *fp;

    fp = opendir(p);         /* open as directory */
    if (fp == NULL)          /* didn't open */
        return(NULL);

    /*
        Now we've got the directory open, set up the
        DIRPATH control block
    */

    dp = (DIRPATH*)calloc(1,sizeof(DIRPATH)); 
    if (dp == NULL)
        return(NULL);             /* couldn't get the space */
    dp->p = fp;                   /* keep the directory file pointer */
    strncpy(dp->path,p,NAMESIZE); /* and the path name */
    dp->path[NAMESIZE-1] = 0;     /* enforce termination */
    return(dp);
}

int dirclose(DIRPATH *dp)
{
    int temp;
    temp = closedir(dp->p);  /* close the directory */
    free(dp);                /* ditch the DIRPATH control block */
    return(temp);            /* return the fclose reply */
}

int dirread(DIRPATH *dp, char *name)
{
    struct dirent *info;
    struct stat   st;
    int           r;
    size_t        n;
    DIR           *d;
    char          *fullname;

    info = readdir(dp->p);    
    if (info == NULL)
        r = DIR_EOF;
    else
    {
        if (strlen(info->d_name) == 0)
        {
       	    strcpy(name, "");
            r = DIR_NULL;
        }
        else
        {
    	    strncpy(name, info->d_name, NAMELTH);
    	    name[NAMELTH-1] = 0;
    	    n = strlen(info->d_name) + strlen(dp->path) + strlen(PATHSEP) + 1;
    	    fullname = (char*)calloc(n, sizeof(char));
    	    if (!fullname)
                error(progname(),
                      "dirread():",
                      "calloc failed to obtain space for pathname");

            strcpy(fullname,dp->path);
            strcat(fullname,PATHSEP);
            strcat(fullname,name);
            if (lstat(fullname,&st)<0)
            	error(progname(),
            	      "dirread(): lstat() failed accessing",
            	      fullname);

            r = DIR_NULL;
            if ((st.st_mode & S_IFMT) == S_IFDIR)
                r = DIR_SUBDIR;
            if ((st.st_mode & S_IFMT) == S_IFREG)
                r = DIR_FILE;
                
            free(fullname);
        }
    }
    return(r);
}

#endif        

